/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.dialogs;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.Layout;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.resources.DCLabels;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.util.Resources;
import net.sf.ext.util.SwingUtil;

/**
 * A normal MessageBox. The message box consists out of an OK button
 * an icon and a message text. The icon depends on the type of message
 * to be displayed (error, warning or information)
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 0.50
 */
public class MessageBox extends JDialog {

    private JTextArea textMessage;
    private JButton buttonClose;
    private JPanel panel = new JPanel(false);
    private final JLabel labelIcon = ComponentFactory.getLabel("");
    
    public  static final int _ERROR = 1;
    public  static final int _WARNING = 2;
    public  static final int _INFORMATION = 3;

    
    /**
     * Initializes the messagebox and shows it
     * @param frame parent frame
     * @param message display String
     * @param type type of the message (see static variables)
     */
    public MessageBox(JFrame frame, String message, int type) {
        super(frame);
        init();
        setModal(true);
        display(message, type);
    }    
    
    /**
     * Initializes the messagebox and shows it
     * @param message display String
     * @param type type of the message (see static variables)
     * @param modal indicates if the messagebox should be shown modal
     */
    public MessageBox(String message, int type, boolean modal) {
        super(SwingUtil.getRootFrame());
        init();
        setModal(modal);
        display(message, type);
    }

    /**
     * Initializes the messagebox and shows it
     * @param message display String
     * @param type type of the message (see static variables)
     */
    public MessageBox(String message, int type) {
        super(SwingUtil.getRootFrame());
        init();
        setModal(true);
        display(message, type);        
    }
    
    /**
     * Closes and disposes this messagebox
     */
    public void close() {
        this.dispose();
    }

    /**
     * Display an icon
     * @param icon
     */
    private void setIcon(ImageIcon icon) {
        labelIcon.setIcon(icon);
    }
    
    /**
     * Shows the messagebox
     * @param message display String
     * @param type type of the message (see static variables)
     */
    private void display(String message, int type) {
        textMessage.setText(message);

        if (type == _ERROR) {
            setIcon(IconLibrary._icoError);
        } else if (type == _WARNING) {
            setIcon(IconLibrary._icoWarning);
        } else {
            setIcon(IconLibrary._icoInformation);
        }

        this.pack();
        this.toFront();
        
        try {
            Dimension size = DataCrow.mainFrame.getSize();
            Point location = DataCrow.mainFrame.getLocation();
            int x = ((location.x + size.width - getSize().width) / 2);
            int y = ((location.y + size.height  - getSize().height) / 2);        
            setLocation(x, y);
        // ignore the errors as the messagebox should always be displayed!
        } catch (Exception ignore) {
            setLocation(150, 150);
        }
        
        buttonClose.requestFocus();
        this.setVisible(true);
    }

    /**
     * Creates the messagebox
     */
    private void init() {
        
        Color color = Settings.getValueAsColor(Repository.settings.stColor);
        this.setBackground(color);
        panel.setBackground(color);
        
        this.setSize(Settings.getValueAsDimension(Repository.settings.stFieldSelectorSize));

        this.getContentPane().add(panel);
        
        this.setResizable(false);
        this.getContentPane().setLayout(Layout.getGBL());

        textMessage = ComponentFactory.getTextArea();
        textMessage.setBackground(color);
        textMessage.setEditable(false);
        JScrollPane scrollIn = new JScrollPane(textMessage);
        scrollIn.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        scrollIn.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
        scrollIn.setPreferredSize(new Dimension(350,50));
        scrollIn.setBorder(null);
        buttonClose = ComponentFactory.getButton(
                      Resources.getText(DCLabels.lblOK), 100);
        buttonClose.addActionListener(new MessageButtonAction());

        panel.setLayout(Layout.getGBL());

        panel.add(labelIcon,  Layout.getGBC( 0, 0, 1, 1, 0.0, 0.0
                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                              new Insets(5, 5, 5, 5), 0, 0));            
        panel.add(scrollIn,   Layout.getGBC( 1, 0, 6, 3, 90.0, 90.0
                             ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                              new Insets(5, 5, 5, 5), 0, 0));
        panel.add(buttonClose, Layout.getGBC(0, 3, 2, 1, 0.0, 0.0
                              ,GridBagConstraints.SOUTHWEST, GridBagConstraints.NONE,
                               new Insets(5, 5, 5, 5), 0, 0));
        this.getContentPane().add(panel);
    }

    private class MessageButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }
}
