/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.modules;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

import net.sf.dc.console.DCMenuBar;
import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.da.ImportCDDialog;
import net.sf.dc.console.software.InsertSoftwarePanel;
import net.sf.dc.console.software.SearchSoftwarePanel;
import net.sf.dc.console.software.SoftwareCategoryForm;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DCFieldDefinition;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.software.Software;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.processes.cdimport.ImportSoftwareCD;
import net.sf.dc.resources.DCLabels;
import net.sf.ext.util.Resources;
import net.sf.ext.util.Utilities;

/**
 * The Software module
 * 
 * @see net.sf.dc.core.modules.DCModules
 * @see net.sf.dc.core.modules.DCModule
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 1.00
 */
public class SoftwareModule extends DCModule {
    
    private final DCMenuBar menuBar;
    
    private final ImportCDDialog importCDDialog = 
        new ImportCDDialog(DataCrow.mainFrame,
                           Resources.getText(DCLabels.lblSoftwareImport));
    
    /**
     * Initializes this module
     */    
    public SoftwareModule() {
        super(DCModules._SOFTWARE,
              Resources.getText(DCLabels.lblModuleSoftware),
              "software",
              "soft",
              "software",
              false,
              true,
              false);
        menuBar = new DCMenuBar(this);        
    }
    
    public DCMenuBar getMenuBar() {
        return menuBar;
    }    
    
    public ImageIcon getIcon() {
        return IconLibrary._icoSoftwareModule;
    }    
    
    public String getObjectName() {
        return Resources.getText(DCLabels.lblSoftware);
    }
    
    public void startImport() {
        importCDDialog.setModule(DCModules.getModule(DCModules._SOFTWARE));
        importCDDialog.setVisible(true);
    }
    
    public JMenu getToolsMenu() {
        JMenu menu = ComponentFactory.getMenu(Resources.getText(DCLabels.lblTools));

        JMenuItem menuCat = ComponentFactory.getMenuItem(
                                                IconLibrary._icoMenuCategory16,
									            Resources.getText(DCLabels.lblCatagoryForm));
        menuCat.addActionListener(new CatagoryFormMenuAction());

        menu.add(menuCat);
        menu.addSeparator();
        
        return menu;
    }      
    
    /**
     * Create the insert and search panel
     */    
    protected void initializeUI() {
        insertPanel = new InsertSoftwarePanel();
        searchPanel = new SearchSoftwarePanel();    
    }
    
    public String getHiddenFieldsSettingsKey() {
        return Repository.settings.stHiddenFieldsSoftware;
    }

    public String getRequiredFieldsSettingsKey() {
        return Repository.settings.stSoftwareRequiredFields;
    }
    
    /**
     * Return the object managed by this module 
     * @see Software
     */    
    public DCObject getDCObject() {
        return new Software();
    }
    
    /**
     * Returns the movie CD import process
     * 
     * @param sourceDir directory which is used as source for the
     *        import
     * @param confirmEachDir indicates whether for each directory
     *        confirmation should be asked if the underlying data
     *        of this directory should be read or not.
     * @param nameOfCD cd name (used as the location name)
     */      
    public ImportCDProcess getCDImportProcess(ImportCDDialog ui,
                                              String sourceDir,
                                              boolean confirmDirs,
                                              boolean recursiveDirs,
                                              String nameOfCD) {
        return new ImportSoftwareCD(ui, sourceDir, confirmDirs, recursiveDirs, nameOfCD);
    }
    
    /**
     * Initalizes all the fields
     */    
    protected void initializeFields() {
        fields = new DCFieldDefinition[20];
        fields[Software._ID] = new DCFieldDefinition(Resources.getText(DCLabels.lblID),
                                        true,
                                        false,
                                        false,
                                        15,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "ID");        
        fields[Software._A_NAME] = new DCFieldDefinition(Resources.getText(DCLabels.lblName),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Name");        
        fields[Software._B_DESCRIPTION] = new DCFieldDefinition(Resources.getText(DCLabels.lblDescription),
                                        false,
                                        true,
                                        false,
                                        8000,
                                        ComponentFactory._LONGTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Description");        
        fields[Software._C_PUBLISHER] = new DCFieldDefinition(Resources.getText(DCLabels.lblPublisher),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Publisher");        
        fields[Software._D_PLATFORM] = new DCFieldDefinition(Resources.getText(DCLabels.lblPlatform),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Platform");     
        fields[Software._E_RELEASEYEAR] = new DCFieldDefinition(Resources.getText(DCLabels.lblReleaseYear),
                                        false,
                                        true,
                                        false,
                                        4,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "ReleaseYear");         
        fields[Software._F_WEBPAGE] = new DCFieldDefinition(Resources.getText(DCLabels.lblWebPage),
                                        false,
                                        false,
                                        false,
                                        255,
                                        ComponentFactory._URLFIELD,
                                        Repository.datatypes._STRING,
                                        "Webpage");        
        fields[Software._G_FILENAME] = new DCFieldDefinition(Resources.getText(DCLabels.lblFileName),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Filename");        
        fields[Software._H_TYPE] = new DCFieldDefinition(Resources.getText(DCLabels.lblType),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SOFTWARECATEGORYCOMBO,
                                        Repository.datatypes._TYPEREFERENCE,
                                        "Type");        
        fields[Software._I_MULTI] = new DCFieldDefinition(Resources.getText(DCLabels.lblMulti),
                                        false,
                                        true,
                                        false,
                                        4,
                                        ComponentFactory._YESNOCOMBO,
                                        Repository.datatypes._STRING,
                                        "Multiplayer");  
        fields[Software._J_PICTUREFRONT] = new DCFieldDefinition(Resources.getText(DCLabels.lblPictureFront),
                                        true,
                                        false,
                                        false,
                                        4000,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "PictureFront");        
        fields[Software._K_PICTUREBACK] = new DCFieldDefinition(Resources.getText(DCLabels.lblPictureBack),
                                        true,
                                        false,
                                        false,
                                        4000,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "PictureBack");        
        fields[Software._L_PICTURECD] = new DCFieldDefinition(Resources.getText(DCLabels.lblPictureCD),
                                        true,
                                        false,
                                        false,
                                        4000,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "PictureCD");        
        fields[Software._M_SCREENSHOTONE] = new DCFieldDefinition(Resources.getText(DCLabels.lblScreenshotOne),
                                        true,
                                        false,
                                        false,
                                        4000,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "ScreenshotOne");        
        fields[Software._N_SCREENSHOTTWO] = new DCFieldDefinition(Resources.getText(DCLabels.lblScreenshotTwo),
                                        true,
                                        false,
                                        false,
                                        4000,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "ScreenshotTwo");        
        fields[Software._O_SCREENSHOTTHREE] = new DCFieldDefinition(Resources.getText(DCLabels.lblScreenshotThree),
                                        true,
                                        false,
                                        false,
                                        4000,
                                        ComponentFactory._PICTUREFIELD,
                                        Repository.datatypes._OBJECT,
                                        "ScreenshotThree");
        fields[Software._P_SERIALKEY] = new DCFieldDefinition(Resources.getText(DCLabels.lblSerialKey),
                                        false,
                                        false,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "SerialKey");     
        fields[Software._Q_FILESIZE] = new DCFieldDefinition(Resources.getText(DCLabels.lblFilesize),
                                        false,
                                        true,
                                        true,
                                        255,
                                        ComponentFactory._NUMBERFIELD,
                                        Repository.datatypes._INTEGER,
                                        "Filesize");         
        fields[Software._R_LOCATION] = new DCFieldDefinition(Resources.getText(DCLabels.lblLocation),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._SHORTTEXTFIELD,
                                        Repository.datatypes._STRING,
                                        "Location"); 
        fields[Software._S_RATING] = new DCFieldDefinition(Resources.getText(DCLabels.lblRating),
                                        false,
                                        true,
                                        false,
                                        255,
                                        ComponentFactory._RATINGCOMBOBOX,
                                        Repository.datatypes._INTEGER,
                                        "Rating");                 
    }
    
    public DCObject[] getDefaultData() throws Exception {
        DCObject[] objects = new DCObject[1];
        
        Software software = new Software();
        String link = "file://" + Utilities.getCurrentDirectory() + File.separator + 
                      "icons_system" + File.separator + "logo.jpg";

        software.setValue(Software._A_NAME, "Data Crow 1.8");
        software.setValue(Software._B_DESCRIPTION, "Data Crow is a music (audio CD's and mp3, ogg, flac, ape, .. files), software and movie database. It is platform independent and easily installable (install and go). Data Crow uses internet services like amazon.com, imdb.com and freedb.org to retrieve infromation for you. Data Crow is easy to use and accessible for anyone. " + 
        		                                   "Data Crow is an opensource project, developed by Robert Jan van der Waals and Jesper Kusk Christensen. The project was started at the beginning of 2002 under the name 'CD Library'. After further development, CD Library was decided to be an obsolete name as it did not cover the functionality and the project was renamed to Data Crow. " +
												   "For those who like to collect software, movies and (or) music (audio files or audio cds), Data Crow can help out. The product needs little input from you to register your collection. For example, registering a piece of Software is as easy as typing the name or the publisher and the product information is retrieved from Amazon. For Audio Files, it is even simpler; browse for the directory containing the audio files and the tag information is read and imported (information about the artist, album name, track name, etc is stored within the music file). An audio CD can simply be put in the CD-rom drive and the information is retrieved for you.");
        software.setValue(Software._S_RATING, new Integer(4));
        software.setValue(Software._D_PLATFORM, "all platforms");
        software.setValue(Software._E_RELEASEYEAR, new Integer(2005));
        software.setValue(Software._F_WEBPAGE, "http://www.datacrow.net");
        software.setValue(Software._G_FILENAME, "/downloads/datacrow.zip");
        software.setValue(Software._P_SERIALKEY, "xxxx-xxxx-xxxx-xxxx");
        software.setValue(Software._R_LOCATION, "CD232");
        software.setValue(Software._Q_FILESIZE, new Integer(4500));
        
        try {
            software.setValue(Software._J_PICTUREFRONT, 
                                        Utilities.fileToBase64String(new URL(link)));
        } catch (Exception ignore) {} // picture error.. skip picture

        objects[0] = software;
        
        return objects;
    }
    
    private class CatagoryFormMenuAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            SoftwareCategoryModule softCatMod = 
                (SoftwareCategoryModule) DCModules.getModule(DCModules._SOFTWARECAT);
            SoftwareCategoryForm categoryForm = softCatMod.getForm(); 
            categoryForm.setVisible(true);
        }
    }
}


